﻿namespace SpikingNeuronNetwork.Lib.Interfaces
{
    using System.Collections.Generic;

    /// <summary>
    /// Spiking Error Interface
    /// </summary>
    public interface ISpikingError
    {
        /// <summary>
        /// Simulation Method Enum
        /// </summary>
        SimulationMethod SimulationMethod { get; set; }
        
        /// <summary>
        /// Get the error between the desired and actual output spike trains
        /// </summary>
        /// <param name="desiredOutputSpikes">The desired output spike train</param>
        /// <param name="actualOutputSpikes">The actual output spike train</param>
        /// <returns>The error</returns>
        double GetError(List<Spike> desiredOutputSpikes, List<Spike> actualOutputSpikes);
        
        /// <summary>
        /// Get the error to output spike time derivatives
        /// </summary>
        /// <param name="desiredOutputSpikes">The desired output spike train</param>
        /// <param name="actualOutputSpikes">The actual output spike train</param>
        /// <returns>A list of the error derivatives</returns>
        List<double> GetErrorToOutputSpikeTimeDerivative(List<Spike> desiredOutputSpikes, List<Spike> actualOutputSpikes);
    }
}
