﻿namespace SpikingNeuronNetwork.Lib
{
    using System.Collections.Generic;
    using System.Linq;
    
    /// <summary>
    /// Neuron Firing History Class
    /// </summary>
    public class NeuronFiringHistory
    {
        /// <summary>
        /// The Neuron Index
        /// </summary>
        public int NeuronIndex { get; set; }

        /// <summary>
        /// The set of spike statistics contained in this neuron firing history
        /// </summary>
        public SortedSet<SpikeStats> SpikeStats { get; set; }

        /// <summary>
        /// The start time of the neuron firing history
        /// </summary>
        public double StartTime 
        {
            get 
            { 
                return (SpikeStats != null && SpikeStats.Count > 0) ? 
                    SpikeStats.First().PreSpikeState.Time :
                    -1; 
            }
        }

        /// <summary>
        /// The end time of the neuron firing history
        /// </summary>
        public double EndTime
        {
            get
            {
                var lastOutputSpikeTime = (OutputSpikes != null && OutputSpikes.Count > 0) ? OutputSpikes.OrderBy(x => x.Time).Last().Time : -1;
                return (SpikeStats != null && SpikeStats.Count > 0) ?
                    (SpikeStats.Last().PostSpikeState.Time > lastOutputSpikeTime ? SpikeStats.Last().PostSpikeState.Time : lastOutputSpikeTime) :
                    -1;
            }
        }

        /// <summary>
        /// Gets or sets a list of output spikes produced during this neuron firing history
        /// </summary>
        public List<Spike> OutputSpikes { get; set; } 

        /// <summary>
        /// Gets a list of indices of neurons that do not produce any output spikes
        /// </summary>
        /// <param name="neuronFiringHistories">A set of neurons' firing histories</param>
        /// <returns>A list of indices of neurons that do not produce any output spikes</returns>
        public static List<int> GetNonFiringNeuronIndices(Dictionary<int, NeuronFiringHistory> neuronFiringHistories)
        {
            return neuronFiringHistories.Where(x => x.Value.OutputSpikes.Count == 0).Select(x => x.Key).ToList();
        }
    }
}
