﻿namespace SpikingNeuronNetwork.Lib
{
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;

    /// <summary>
    /// Spike set class
    /// </summary>
    public class SpikeSet
    {
        /// <summary>
        /// Gets or sets the identifier.
        /// </summary>
        /// <value>
        /// The identifier.
        /// </value>
        public int Id { get; set; }

        /// <summary>
        /// Gets or sets the input spikes.
        /// </summary>
        /// <value>
        /// The input spikes.
        /// </value>
        public List<Spike> InputSpikes { get; set; }

        /// <summary>
        /// Gets or sets the output spikes.
        /// </summary>
        /// <value>
        /// The output spikes.
        /// </value>
        public List<Spike> OutputSpikes { get; set; }

        /// <summary>
        /// Gets the spike times string.
        /// </summary>
        /// <param name="spikes">The spikes.</param>
        /// <returns>The spike times string</returns>
        public static string GetSpikeTimesString(IEnumerable<Spike> spikes)
        {
            var spikeStringBuilder = new StringBuilder();
            spikes.ToList().ForEach(x => spikeStringBuilder.AppendLine("\t" + x.Time + "(Neuron: " + x.NeuronIndex + ")"));
            return spikeStringBuilder.ToString();
        }

        /// <summary>
        /// Returns a <see cref="System.String" /> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String" /> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return "Input Spikes:\n" + GetSpikeTimesString(InputSpikes) + "Output Spikes:\n" + GetSpikeTimesString(OutputSpikes);
        }
    }
}
