﻿namespace SpikingNeuronNetwork.Lib
{
    using System;
    using System.Text;
    
    /// <summary>
    /// The spike statistics class
    /// </summary>
    public class SpikeStats : IComparable
    {
        /// <summary>
        /// Gets or sets the neuron state directly before the spike occurs
        /// </summary>
        public NeuronState PreSpikeState { get; set; }

        /// <summary>
        /// Gets or sets the neuron state directly after the spike occurs
        /// </summary>
        public NeuronState PostSpikeState { get; set; }

        /// <summary>
        /// Gets or sets the synaptic weight
        /// </summary>
        public double Weight { get; set; }

        /// <summary>
        /// Gets or sets the synapse
        /// </summary>
        public Synapse Synapse { get; set; }

        /// <summary>
        /// Returns a <see cref="System.String" /> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String" /> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            var stringBuilder = new StringBuilder();
            stringBuilder.AppendLine("--->      Spike Time: " + Synapse + " at " + PreSpikeState.Time);
            stringBuilder.AppendLine("--->Pre -Spike Phase: " + PreSpikeState.StateVariable);
            stringBuilder.AppendLine("--->Post-Spike Phase: " + PostSpikeState.StateVariable);
            stringBuilder.Append("--->Weight: " + Weight);
            return stringBuilder.ToString();
        }

        /// <summary>
        /// Compares the current instance with another object of the same type and returns an integer that indicates whether the current instance precedes, follows, or occurs in the same position in the sort order as the other object.
        /// </summary>
        /// <param name="obj">An object to compare with this instance.</param>
        /// <returns>
        /// A value that indicates the relative order of the objects being compared. The return value has these meanings: Value Meaning Less than zero This instance precedes <paramref name="obj" /> in the sort order. Zero This instance occurs in the same position in the sort order as <paramref name="obj" />. Greater than zero This instance follows <paramref name="obj" /> in the sort order.
        /// </returns>
        /// <exception cref="System.ArgumentException">Object is not a SpikeStats</exception>
        public int CompareTo(object obj)
        {
            if (obj == null) return 1;

            var otherSpikeStats = obj as SpikeStats;
            if (otherSpikeStats != null)
                return PreSpikeState.Time.CompareTo(otherSpikeStats.PreSpikeState.Time);
            throw new ArgumentException("Object is not a SpikeStats");
        }
    }
}
