﻿namespace SpikingNeuronNetwork.Lib
{
    /// <summary>
    /// Synapse Class
    /// </summary>
    public class Synapse
    {
        /// <summary>
        /// Gets or sets the index of the input neuron.
        /// </summary>
        /// <value>
        /// The index of the input neuron.
        /// </value>
        public int InputNeuronIndex { get; set; }

        /// <summary>
        /// Gets or sets the index of the output neuron.
        /// </summary>
        /// <value>
        /// The index of the output neuron.
        /// </value>
        public int OutputNeuronIndex { get; set; }

        /// <summary>
        /// Creates a new instance of the synapse class
        /// </summary>
        /// <param name="inputNeuronIndex">The input neuron index</param>
        /// <param name="outputNeuronIndex">The output neuron index</param>
        public Synapse(int inputNeuronIndex, int outputNeuronIndex)
        {
            InputNeuronIndex = inputNeuronIndex;
            OutputNeuronIndex = outputNeuronIndex;
        }

        /// <summary>
        /// Returns a <see cref="System.String" /> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String" /> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return "(" + InputNeuronIndex + "->" + OutputNeuronIndex + ")";
        }

        /// <summary>
        /// Returns a bool indicating whether the synapse is equal to an object
        /// </summary>
        /// <param name="obj">The object</param>
        /// <returns>A bool indicating whether the synapse is equal to an object</returns>
        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != GetType()) return false;
            return ((Synapse)obj).InputNeuronIndex == InputNeuronIndex && ((Synapse)obj).OutputNeuronIndex == OutputNeuronIndex;
        }

        /// <summary>
        /// Returns a hash code representing the synapse
        /// </summary>
        /// <returns>A hash code representing the synapse</returns>
        public override int GetHashCode()
        {
            unchecked
            {
                return (InputNeuronIndex * 397) ^ OutputNeuronIndex;
            }
        }
    }
}
