﻿namespace SpikingNeuronNetwork.Lib.Training
{
    using Interfaces;
    using System.Collections.Generic;
    using System.Linq;

    /// <summary>
    /// Error Derivative Parameters Class
    /// </summary>
    public class ErrorDerivativeParameters : IErrorDerivativeParameters
    {
        /// <summary>
        /// Gets or sets the error to output spike time derivatives dictionary
        /// </summary>
        public Dictionary<int, double> ErrorToOutputSpikeTimeDerivatives { get; set; }
        
        /// <summary>
        /// Gets or sets the output spike time to weight derivatives dictionary
        /// </summary>
        public Dictionary<Synapse, double> OutputSpikeTimeToWeightDerivatives { get; set; }

        /// <summary>
        /// Creates a new instance of ErrorDerivativeParameters
        /// </summary>
        public ErrorDerivativeParameters()
        {
            ErrorToOutputSpikeTimeDerivatives = new Dictionary<int, double>();
            OutputSpikeTimeToWeightDerivatives = new Dictionary<Synapse, double>();
        }

        /// <summary>
        /// Gets a list of the synapses for which there are error derivative parameters
        /// </summary>
        /// <returns>The list of the synapses for which there are error derivative parameters</returns>
        public List<Synapse> GetSynapses()
        {
            return OutputSpikeTimeToWeightDerivatives.Keys.ToList();
        }

        /// <summary>
        /// Get the derivate of the error relative change in weight for a certain synapse
        /// </summary>
        /// <param name="synapse">The synapse</param>
        /// <returns>The error derivative</returns>
        public double GetErrorDerivative(Synapse synapse)
        {
            return ErrorToOutputSpikeTimeDerivatives[synapse.OutputNeuronIndex]*
                   OutputSpikeTimeToWeightDerivatives[synapse];
        }
    }
}
