﻿namespace SpikingNeuronNetwork.Lib.Training
{
    using Interfaces;

    /// <summary>
    /// Steepest Descent Training With Momentum Class, inherits from ITrainingAlgorithm
    /// </summary>
    public class SteepestDescentWithMomentumTraining : ITrainingAlgorithm
    {
        /// <summary>
        /// Update the weights in the current network according to the training algorithm
        /// </summary>
        /// <param name="trainingStats">The training statistic parameters</param>
        /// <param name="errorDerivativeParameters">The current error derivative parameters</param>
        /// <param name="previousErrorDerivativeParameters">The previous error derivative parameters</param>
        public void UpdateWeights(TrainingStats trainingStats, IErrorDerivativeParameters errorDerivativeParameters,
                                  IErrorDerivativeParameters previousErrorDerivativeParameters)
        {
            foreach (var synapse in errorDerivativeParameters.GetSynapses())
            {
                trainingStats.CurrentNetwork.SetSynapticWeight(synapse,
                                        trainingStats.CurrentNetwork.GetSynapticWeight(synapse) -
                                        trainingStats.LearningRate * errorDerivativeParameters.GetErrorDerivative(synapse) -
                                        trainingStats.MomentumLearningRate * (previousErrorDerivativeParameters == null ? 0 : previousErrorDerivativeParameters.GetErrorDerivative(synapse)));
            }
        }
    }
}
