﻿namespace SpikingNeuronNetwork.Lib.Training
{
    using Interfaces;
    using System.Collections.Generic;

    /// <summary>
    /// Training Statistics Class
    /// </summary>
    public class TrainingStats
    {
        /// <summary>
        /// Gets or sets training Stats Id
        /// </summary>
        public int Id { get; set; }

        /// <summary>
        /// Gets or sets the original network before training
        /// </summary>
        public SpikingNeuronNetwork OriginalNetwork { get; set; }

        /// <summary>
        /// Gets or sets the training method
        /// </summary>
        public TrainingMethod TrainingMethod { get; set; }

        /// <summary>
        /// Gets or sets the training algorithm
        /// </summary>
        public ITrainingAlgorithm TrainingAlgorithm { get; set; }

        /// <summary>
        /// Gets or sets the learning rate
        /// </summary>
        public double LearningRate { get; set; }

        /// <summary>
        /// Gets or sets the momentum learning rate
        /// </summary>
        public double MomentumLearningRate { get; set; }

        /// <summary>
        /// Gets or sets the maximum error after training
        /// </summary>
        public double MaximumErrorAfterTraining { get; set; }

        /// <summary>
        /// Gets or sets the maximum number of training epochs
        /// </summary>
        public double MaximumNumberOfTrainingEpochs { get; set; }

        /// <summary>
        /// Gets or sets the training spike set, including input spikes and desired output spikes
        /// </summary>
        public List<SpikeSet> TrainingSet { get; set; }

        /// <summary>
        /// Gets or sets the error calculator
        /// </summary>
        public ISpikingError ErrorCalculator { get; set; }

        /// <summary>
        /// Gets or sets the current number of epochs
        /// </summary>
        public int NumEpochs { get; set; }

        /// <summary>
        /// Gets or sets the training stats on a per epoch basis
        /// </summary>
        public Dictionary<int, TrainingStatsPerEpoch> PerEpochStats { get; set; }

        /// <summary>
        /// Gets or sets the current network after training
        /// </summary>
        public SpikingNeuronNetwork CurrentNetwork { get; set; }

        /// <summary>
        /// Resets training to its initial untrained state
        /// </summary>
        public void ResetTraining()
        {
            CurrentNetwork = OriginalNetwork.Clone();
            NumEpochs = 0;
            PerEpochStats = new Dictionary<int, TrainingStatsPerEpoch>();            
        }
    }
}
