﻿namespace SpikingNeuronNetwork.Lib.Training
{
    using System;
    using System.Collections.Generic;

    /// <summary>
    /// Training Stats Per Epoch Class, inherits from a tuple representing the error and non-firing neuron indices
    /// </summary>
    public class TrainingStatsPerEpoch : Tuple<double, List<int>>
    {
        /// <summary>
        /// Creates a new TrainingStatsPerEpoch instance
        /// </summary>
        /// <param name="error">The error</param>
        /// <param name="nonFiringNeuronIndices">The indices of non-firing neurons</param>
        public TrainingStatsPerEpoch(double error, List<int> nonFiringNeuronIndices) : base(error, nonFiringNeuronIndices)
        {
        }

        /// <summary>
        /// Creates a string representation of the TrainingStatsPerEpoch object
        /// </summary>
        /// <returns>The string representation of a TrainingStatsPerEpoch object</returns>
        public override string ToString()
        {
            return "Error of " + Item1 + ((Item2.Count > 0) ? " with Neuron(s) " + String.Join(",", Item2) + " Not Firing" : " with All Neurons Firing");
        }
    }
}
