﻿namespace SpikingNeuronNetwork.Test
{
    using Lib;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using System;
    using System.Collections.Generic;

    /// <summary>
    /// Tests the sum of squared error calculation between spike trains
    /// </summary>
    [TestClass]
    public class SumOfSquaredSpikingErrorTests
    {
        /// <summary>
        /// Tests the sum of squared error calculation between spike trains
        /// containing one spike
        /// </summary>
        [TestMethod]
        public void SumOfSquaredSpikingErrorOneOutputNeuronTest()
        {
            var desiredSpikeTimes = new List<Spike>
                {
                    new Spike {NeuronIndex = 0, Time = 1}
                };
            var actualSpikeTimes = new List<Spike>
                {
                    new Spike {NeuronIndex = 0, Time = 1.1}
                };

            SumOfSquareSpikingErrorTestHelper(desiredSpikeTimes, actualSpikeTimes, 0.005);
        }

        /// <summary>
        /// Tests the sum of squared error calculation between spike trains
        /// containing two spikes
        /// </summary>
        [TestMethod]
        public void SumOfSquaredSpikingErrorTwoOutputNeuronsTest()
        {
            var desiredSpikeTimes = new List<Spike>
                {
                    new Spike {NeuronIndex = 0, Time = 1},
                    new Spike {NeuronIndex = 1, Time = 3}
                };
            var actualSpikeTimes = new List<Spike>
                {
                    new Spike {NeuronIndex = 0, Time = 1.1},
                    new Spike {NeuronIndex = 1, Time = 2.8}
                };

            SumOfSquareSpikingErrorTestHelper(desiredSpikeTimes, actualSpikeTimes, 0.025);
        }

        #region Helper Methods

        /// <summary>
        /// Helper method to test sum of square error calculation, including its derivative calculation
        /// </summary>
        /// <param name="desiredSpikeTimes">Set of desired spikes</param>
        /// <param name="actualSpikeTimes">Set of actual spikes</param>
        /// <param name="expectedError">The expected error between the two spike trains</param>
        private static void SumOfSquareSpikingErrorTestHelper(List<Spike> desiredSpikeTimes, List<Spike> actualSpikeTimes, double expectedError)
        {
            var errorCalculator = new SumOfSquaredSpikingError();
            var error = errorCalculator.GetError(desiredSpikeTimes, actualSpikeTimes);
            Assert.IsTrue(Math.Abs(error - expectedError) < TestHelper.Epsilon);
            errorCalculator.SimulationMethod = SimulationMethod.EventDriven;
            var errorDerivativesEventDriven = errorCalculator.GetErrorToOutputSpikeTimeDerivative(desiredSpikeTimes, actualSpikeTimes);
            errorCalculator.SimulationMethod = SimulationMethod.Numerical;
            var errorDerivativesNumerical = errorCalculator.GetErrorToOutputSpikeTimeDerivative(desiredSpikeTimes, actualSpikeTimes);
            Assert.AreEqual(errorDerivativesEventDriven.Count, errorDerivativesNumerical.Count);
            for (var j = 0; j < errorDerivativesEventDriven.Count; j++)
            {
                Assert.IsTrue(Math.Abs(errorDerivativesEventDriven[j] - errorDerivativesNumerical[j]) < TestHelper.Epsilon);
            }
        }

        #endregion
    }
}
