﻿namespace SpikingNeuronNetwork.Test
{
    using Lib;
    using Lib.NeuronModels;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using System;
    using System.Collections.Generic;
    using System.Linq;

    /// <summary>
    /// Tests theta neuron functionality
    /// </summary>
    [TestClass]
    public class ThetaNeuronTests
    {
        public static List<Spike> InputSpikeTimes = new List<Spike>
                {
                    new Spike { Time = 2, NeuronIndex = 0}, 
                    new Spike { Time = 1, NeuronIndex = 1}, 
                    new Spike { Time = 1.5, NeuronIndex = 2}, 
                    new Spike { Time = 4, NeuronIndex = 3}, 
                };

        /// <summary>
        /// Tests numerical and event-driven results are equivalent for a
        /// theta neuron with 4 inputs and random weights
        /// </summary>
        [TestMethod]
        public void ThetaNeuronRandomWeightsMethodTest()
        {
            // Construct theta neuron object with randomized weights
            const int numInputs = 4;
            var thetaNeuron = new ThetaNeuron(numInputs);
            TestHelper.CompareNumericalAndEventDrivenSpikeTrains(thetaNeuron, InputSpikeTimes);
        }

        /// <summary>
        /// Tests numerical and event-driven results are equivalent for a
        /// theta neuron with 4 inputs and random weights
        /// </summary>
        [TestMethod]
        public void ThetaNeuronIdenticalPositiveIoPositiveWeightsMethodTest()
        {
            // Construct theta neuron object with randomized weights
            const int numInputs = 4;
            var thetaNeuron = new ThetaNeuron(numInputs, 0.2, 0.01);
            TestHelper.CompareNumericalAndEventDrivenSpikeTrains(thetaNeuron, InputSpikeTimes);
        }

        [TestMethod]
        public void ThetaNeuronIdenticalPositiveIoNegativeWeightsMethodTest()
        {
            // Construct theta neuron object with randomized weights
            const int numInputs = 4;
            var thetaNeuron = new ThetaNeuron(numInputs, -0.001, 0.01);
            TestHelper.CompareNumericalAndEventDrivenSpikeTrains(thetaNeuron, InputSpikeTimes);
        }
        
        [TestMethod]
        public void ThetaNeuronIdenticalPositiveIoRandomizedWeightsMethodTest()
        {
            // Construct theta neuron object with randomized weights
            const int numInputs = 4;
            var thetaNeuron = new ThetaNeuron(numInputs, 1e6, 0.01);
            TestHelper.CompareNumericalAndEventDrivenSpikeTrains(thetaNeuron, InputSpikeTimes);
        }

        [TestMethod]
        public void ThetaNeuronIdenticalPositiveWeightsMethodTest()
        {
            // Construct theta neuron object with randomized weights
            const int numInputs = 4;
            var thetaNeuron = new ThetaNeuron(numInputs, 0.01);
            TestHelper.CompareNumericalAndEventDrivenSpikeTrains(thetaNeuron, InputSpikeTimes);
        }

        [TestMethod]
        public void ThetaNeuronIdenticalNegativeWeightsMethodTest()
        {
            // Construct theta neuron object with randomized weights
            const int numInputs = 4;
            var thetaNeuron = new ThetaNeuron(numInputs, -0.01);
            TestHelper.CompareNumericalAndEventDrivenSpikeTrains(thetaNeuron, InputSpikeTimes);
        }

        [TestMethod]
        public void ThetaNeuronStackedTest()
        {
            for (var j2 = 0; j2 < 1000; j2++)
            {
                const int numInputs = 4;
                const int maxNumOutputSpikes = 3;

                // Construct theta neuron object with randomized weights
                var thetaNeuron = new ThetaNeuron(numInputs);

                // Display Neuron Properties
                Console.WriteLine(thetaNeuron);

                //Simulate Theta Neuron with Event-Driven Methodology
                thetaNeuron.Method = SimulationMethod.EventDriven;
                var outputSpikeTimesEventDriven =
                    thetaNeuron.RunThetaNeuron(InputSpikeTimes, maxNumOutputSpikes).ToList();

                //Stack two neurons sequentially
                if (outputSpikeTimesEventDriven.Count <= 0) return;
                Console.WriteLine("A second neuron is simulated in connection to the output of the previous neuron");
                var thetaNeuron2 = new ThetaNeuron(1);
                var outputSpikeTimesList =
                    outputSpikeTimesEventDriven.Select((t, j) => new Spike {Time = t.Time, NeuronIndex = j}).ToList();
                var outputSpikeTimesEventDriven2 =
                    thetaNeuron2.RunThetaNeuron(outputSpikeTimesList, maxNumOutputSpikes).ToList();
                if (outputSpikeTimesEventDriven2.Count > 0)
                {
                    Assert.IsTrue(outputSpikeTimesEventDriven.First().Time < outputSpikeTimesEventDriven2.First().Time);
                }
            }
        }
    }
}
